import re

mappings = {
  "CAN_OPTIMIZE": "bool", 
  "CLEAN_KEYS": "bool", 
  "FORMAT": "string", 
  "KEEP_EMPTY_VALS": "bool", 
  "MV_ADD": "bool", 
  "REGEX": "string", 
  "SOURCE_KEY": "string", 
  "SPLUNK_DB": "string", 
  "SSL": "bool", 
  "__conf": "string", 
  "__stanza": "string", 
  "action": "string", 
  "action.*": "string", 
  "action.email": "bool", 
  "action.email*": "string", 
  "action.email.auth_password": "string", 
  "action.email.auth_username": "string", 
  "action.email.bcc": "string", 
  "action.email.cc": "string", 
  "action.email.command": "string", 
  "action.email.format": "string", 
  "action.email.from": "string", 
  "action.email.hostname": "string", 
  "action.email.inline": "bool", 
  "action.email.mailserver": "string", 
  "action.email.maxresults": "number", 
  "action.email.maxtime": "number", 
  "action.email.pdfview": "string", 
  "action.email.preprocess_results": "string", 
  "action.email.reportPaperOrientation": "string", 
  "action.email.reportPaperSize": "string", 
  "action.email.reportServerEnabled": "bool", 
  "action.email.reportServerURL": "string", 
  "action.email.sendpdf": "bool", 
  "action.email.sendresults": "bool", 
  "action.email.subject": "string", 
  "action.email.to": "string", 
  "action.email.track_alert": "bool", 
  "action.email.ttl": "number", 
  "action.email.use_ssl": "bool", 
  "action.email.use_tls": "bool", 
  "action.populate_lookup": "bool", 
  "action.populate_lookup.command": "string", 
  "action.populate_lookup.dest": "string", 
  "action.populate_lookup.hostname": "string", 
  "action.populate_lookup.maxresults": "number", 
  "action.populate_lookup.maxtime": "number", 
  "action.populate_lookup.track_alert": "bool", 
  "action.populate_lookup.ttl": "number", 
  "action.rss": "bool", 
  "action.rss.command": "string", 
  "action.rss.hostname": "string", 
  "action.rss.maxresults": "number", 
  "action.rss.maxtime": "number", 
  "action.rss.track_alert": "bool", 
  "action.rss.ttl": "number", 
  "action.script": "bool", 
  "action.script.command": "string", 
  "action.script.filename": "string", 
  "action.script.hostname": "string", 
  "action.script.maxresults": "number", 
  "action.script.maxtime": "number", 
  "action.script.track_alert": "bool", 
  "action.script.ttl": "number", 
  "action.summary_index": "bool", 
  "action.summary_index._name": "string", 
  "action.summary_index.command": "string", 
  "action.summary_index.hostname": "string", 
  "action.summary_index.inline": "bool", 
  "action.summary_index.maxresults": "number", 
  "action.summary_index.maxtime": "number", 
  "action.summary_index.track_alert": "bool", 
  "action.summary_index.ttl": "number", 
  "actions": "string", 
  "add": "string", 
  "alert.digest_mode": "bool", 
  "alert.expires": "number", 
  "alert.severity": "string", 
  "alert.suppress": "bool", 
  "alert.suppress.fields": "string", 
  "alert.suppress.period": "number", 
  "alert.track": "string", 
  "alert_comparator": "string", 
  "alert_condition": "string", 
  "alert_threshold": "number", 
  "alert_type": "string", 
  "alias.*": "string", 
  "append_slaves": "bool", 
  "args.*": "string", 
  "assureUTF8": "bool", 
  "author": "string", 
  "autoAddServers": "bool", 
  "autoLB": "bool", 
  "auto_cancel": "number", 
  "auto_finalize_ec": "number", 
  "auto_pause": "number", 
  "backoffAtStartup": "number", 
  "baseline": "number", 
  "blacklist": "string", 
  "blacklist.": "string", 
  "blacklist.0": "string", 
  "blacklist.1": "string", 
  "blacklist.2": "string", 
  "blacklist.3": "string", 
  "blacklist.4": "string", 
  "blacklist.5": "string", 
  "blacklist.6": "string", 
  "blacklist.7": "string", 
  "blacklist.8": "string", 
  "blacklist.9": "string", 
  "blacklistNames": "string", 
  "blacklistURLs": "string", 
  "blockOnQueueFull": "bool", 
  "blockSignSize": "number", 
  "capabilities": "string", 
  "check-index": "bool", 
  "check-new": "bool", 
  "check-path": "bool", 
  "checkTimedOutServersFrequency": "number", 
  "check_for_updates": "bool", 
  "classes": "string", 
  "coldPath": "string", 
  "coldToFrozenDir": "string", 
  "coldToFrozenScript": "string", 
  "compressRawdata": "bool", 
  "compressed": "bool", 
  "configured": "bool", 
  "connectionTimeout": "number", 
  "connection_host": "string", 
  "continueMatching": "bool", 
  "counters": "string", 
  "crc-salt": "string", 
  "createrole": "string", 
  "cron_schedule": "string", 
  "defaultApp": "string", 
  "defaultGroup": "string", 
  "default_match": "string", 
  "delete": "string", 
  "description": "string", 
  "disabled": "bool", 
  "dispatch.*": "string", 
  "dispatch.buckets": "number", 
  "dispatch.earliest_time": "string", 
  "dispatch.latest_time": "string", 
  "dispatch.lookups": "bool", 
  "dispatch.max_count": "number", 
  "dispatch.max_time": "number", 
  "dispatch.reduce_freq": "number", 
  "dispatch.rt_backfill": "bool", 
  "dispatch.spawn_process": "bool", 
  "dispatch.time_format": "string", 
  "dispatch.ttl": "number", 
  "displayview": "string", 
  "dropEventsOnQueueFull": "number", 
  "eai:data": "string", 
  "earliest_time": "string", 
  "email": "string", 
  "enableOnlineBucketRepair": "bool", 
  "enableSplunkWebSSL": "bool", 
  "enable_lookups": "bool", 
  "endpoint": "string", 
  "exec_mode": "string", 
  "external_cmd": "string", 
  "fields": "string", 
  "fields_list": "string", 
  "filename": "string", 
  "filterType": "string", 
  "followTail": "bool", 
  "force_bundle_replication": "bool", 
  "frozenTimePeriodInSecs": "number", 
  "heartbeatFrequency": "number", 
  "heartbeatMcastAddr": "string", 
  "heartbeatPort": "string", 
  "hive": "string", 
  "homePath": "string", 
  "host": "string", 
  "host_regex": "string", 
  "host_segment": "number", 
  "hosts": "string", 
  "httpport": "string", 
  "id": "string", 
  "ignore-older-than": "string", 
  "imported_roles": "string", 
  "index": "string", 
  "indexAndForward": "bool", 
  "initialBackoff": "number", 
  "instances": "string", 
  "interval": "number", 
  "is_active": "bool", 
  "is_scheduled": "bool", 
  "is_visible": "bool", 
  "label": "string", 
  "latest_time": "string", 
  "level": "string", 
  "logs": "string", 
  "lookup.field.input.*": "string", 
  "lookup.field.output.*": "string", 
  "lookup_host": "string", 
  "manageable": "bool", 
  "maxBackoff": "number", 
  "maxBloomBackfillBucketAge": "number", 
  "maxConcurrentOptimizes": "number", 
  "maxDataSize": "number", 
  "maxHotBuckets": "number", 
  "maxHotIdleSecs": "number", 
  "maxHotSpanSecs": "number", 
  "maxMemMB": "number", 
  "maxMetaEntries": "number", 
  "maxNumberOfRetriesAtHighestBackoff": "number", 
  "maxQueueSize": "number", 
  "maxRunningProcessGroups": "number", 
  "maxRunningProcessGroupsLowPriority": "number", 
  "maxTotalDataSizeMB": "number", 
  "maxWarmDBCount": "number", 
  "max_concurrent": "number", 
  "max_count": "number", 
  "max_matches": "number", 
  "max_offset_secs": "number", 
  "max_time": "number", 
  "method": "string", 
  "mgmtHostPort": "string", 
  "minFreeSpace": "number", 
  "minRawFileSyncSecs": "number", 
  "min_matches": "number", 
  "min_offset_secs": "number", 
  "monitorSubnodes": "number", 
  "monitorSubtree": "number", 
  "name": "string", 
  "namespace": "string", 
  "next_scheduled_time": "string", 
  "no_appending_timestamp": "bool", 
  "no_priority_stripping": "bool", 
  "nonce": "string", 
  "now": "string", 
  "object": "string", 
  "overwrite": "bool", 
  "partialServiceMetaPeriod": "number", 
  "pass4SymmKey": "string", 
  "passAuth": "string", 
  "password": "string", 
  "payload": "string", 
  "peername": "string", 
  "port": "number", 
  "priority": "number", 
  "proc": "string", 
  "qualifiedSearch": "string", 
  "quarantineFutureSecs": "number", 
  "quarantinePastSecs": "number", 
  "queue": "string", 
  "quota": "string", 
  "rawChunkSizeBytes": "number", 
  "realm": "string", 
  "realname": "string", 
  "realtime_schedule": "bool", 
  "receiveTimeout": "number", 
  "recursive": "bool", 
  "reduce_freq": "number", 
  "reload_macros": "bool", 
  "remotePassword": "string", 
  "remoteUsername": "string", 
  "remote_server_list": "string", 
  "removedTimedOutServers": "bool", 
  "rename-source": "string", 
  "repositoryLocation": "string", 
  "request.ui_dispatch_app": "string", 
  "request.ui_dispatch_view": "string", 
  "requireClientCert": "bool", 
  "required_field_list": "string", 
  "restart_background_jobs": "bool", 
  "restart_on_searchpeer_add": "bool", 
  "restrictToHost": "string", 
  "rf": "string", 
  "roles": "string", 
  "rootCA": "string", 
  "rotatePeriodInSecs": "number", 
  "rtSrchJobsQuota": "number", 
  "rt_blocking": "bool", 
  "rt_indexfilter": "bool", 
  "rt_maxblocksecs": "number", 
  "rt_queue_size": "number", 
  "run_on_startup": "bool", 
  "script": "string", 
  "search": "string", 
  "search_listener": "string", 
  "search_mode": "string", 
  "sendCookedData": "bool", 
  "sendTimeout": "number", 
  "server": "string", 
  "serverCert": "string", 
  "serverName": "string", 
  "serverTimeout": "number", 
  "servers": "string", 
  "serviceMetaPeriod": "number", 
  "sessionTimeout": "string", 
  "shareBundles": "bool", 
  "sig": "string", 
  "skipOurselves": "bool", 
  "slaves": "string", 
  "source": "string", 
  "sourcetype": "string", 
  "spawn_process": "bool", 
  "srchDiskQuota": "number", 
  "srchFilter": "string", 
  "srchIndexesAllowed": "string", 
  "srchIndexesDefault": "string", 
  "srchJobsQuota": "number", 
  "srchTimeWin": "number", 
  "sslAltNameToCheck": "string", 
  "sslCertPath": "string", 
  "sslCipher": "string", 
  "sslCommonNameToCheck": "string", 
  "sslPassword": "string", 
  "sslRootCAPath": "string", 
  "sslVerifyServerCert": "bool", 
  "stack_id": "string", 
  "stanza": "string", 
  "startingNode": "string", 
  "startwebserver": "bool", 
  "statusTimeout": "number", 
  "status_buckets": "number", 
  "suppressBannerList": "string", 
  "syncMeta": "bool", 
  "sync_bundle_replication": "bool", 
  "tags": "string", 
  "targetDc": "string", 
  "targetRepositoryLocation": "string", 
  "targetUri": "string", 
  "template": "string", 
  "thawedPath": "string", 
  "throttleCheckPeriod": "number", 
  "time-before-close": "number", 
  "time_field": "string", 
  "time_format": "string", 
  "timeout": "number", 
  "timestampformat": "string", 
  "tmpFolder": "string", 
  "transform": "string", 
  "trustedIP": "string", 
  "ts": "number", 
  "ttl": "number", 
  "type": "string", 
  "tz": "string", 
  "update": "bool", 
  "userid": "string", 
  "username": "string", 
  "value": "string", 
  "version": "string", 
  "visible": "bool", 
  "vsid": "string", 
  "whitelist": "string", 
  "whitelist.": "string", 
  "whitelist.0": "string", 
  "whitelist.1": "string", 
  "whitelist.2": "string", 
  "whitelist.3": "string", 
  "whitelist.4": "string", 
  "whitelist.5": "string", 
  "whitelist.6": "string", 
  "whitelist.7": "string", 
  "whitelist.8": "string", 
  "whitelist.9": "string",
  
  
  'can_change_perms': "bool", 
  'can_share_global': "bool", 
  'modifiable': "bool", 
  'can_share_user': "bool", 
  'can_share_app': "bool", 
  'can_write': "bool",
  
  'eventIsStreaming': "bool",
  'eventIsTruncated': "bool",
  'resultIsStreaming': "bool",
  
  'action.email': "bool",
  'action.populate_lookup': "bool",
  'action.rss': "bool",
  'action.script': "bool",
  'action.summary_index': "bool",
  'dispatch.spawn_process': "bool",
  'restart_on_searchpeer_add': "bool",
  'run_on_startup': "bool",
  
  #"eai:acl.can_write": "bool",
  #"eai:acl.modifiable": "bool",
  
  'alert.severity': "number",
  'dispatch.buckets': "number",
  'dispatch.max_count': "number",
  'dispatch.max_time': "number",
  'dispatch.reduce_freq': "number",
  'max_concurrent': "number",
  
  
  'diskUsage': "number",
  'dropCount': "number",
  'eventAvailableCount': "number",
  'eventCount': "number",
  'eventFieldCount': "number",
  'numPreviews': "number",
  'priority': "number",
  'resultCount': "number",
  'resultPreviewCount': "number",
  'scanCount': "number",
  'statusBuckets': "number",
  'ttl': "number",
  
  'doneProgress': "number",
  'runDuration': "number",
  'searchEarliestTime': "number",
  'searchLatestTime': "number",
  
  "diskUsage": "number",
  "numPreviews": "number",
  "priority": "number",
  "statusBuckets": "number",
  "ttl": "number",
  "doneProgress": "number",
  "runDuration": "number",
  
  "enableRealtimeSearch": "bool", 
  
  "__offset": "number",
  
  "eventsStreaming": "bool",
  "eventsTimeOrdered": "bool",
  "remoteTimeOrdered": "bool",
  
  "data.count": "number",
  "data.operator": "bool",
  
  "bytes": "number"  
}

bool_regexes = [
  re.compile(r"^is.*"),
  re.compile(r"^is_.*"),
  re.compile(r"^can_.*"),
  re.compile(r"^disabled.*"),
  re.compile(r"^.*Is.*"),
]

number_regexes = [
  re.compile(r"^.*Count$"),
  re.compile(r"^performance\.*"),
]